!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! !ROUTINE: timeinterp_RFE2gdas
! \label{timeinterp_RFE2gdas}
!
! !REVISION HISTORY:
!
! 2 June 2010: Soni Yatheendradas; Initial LIS version for FEWSNET
! 8 Mar 2012: Updated for LIS7
!
! !INTERFACE:
subroutine timeinterp_RFE2gdas(n,findex)
! !USES:
  use ESMF
  use LIS_coreMod, only : LIS_rc,LIS_domain
  use LIS_metforcingMod, only : LIS_FORC_Base_State, LIS_forc
  use LIS_FORC_AttributesMod
  use LIS_logMod, only : LIS_verify,LIS_logunit
  use RFE2gdas_forcingMod, only : RFE2gdas_struc

  implicit none

! !ARGUMENTS:
  integer, intent(in) :: n
  integer, intent(in) :: findex
!
! !DESCRIPTION:
!  Temporally interpolates the forcing data to the current model
!  timestep (evenly distributed over sub-daily timesteps for now). 
!
!  The arguments are:
!  \begin{description}
!  \item[n]
!    index of the nest
!  \item[findex]
!    index of the supplemental forcing source
!  \item[suppdata1]
!    previous forcing values
!  \item[suppdata2]
!    next forcing values
!  \end{description}
!
!EOP

  integer :: t,index1
  
  integer          :: status
  type(ESMF_Field) :: pcpField, cpcpField
  real, pointer    :: pcp(:), cpcp(:)
  real,  allocatable :: ratio(:)


  call ESMF_StateGet(LIS_FORC_Base_State(n,findex),&
       trim(LIS_FORC_Rainf%varname(1)),pcpField,&
       rc=status)
  call LIS_verify(status, 'Error: enable Rainf in forcing variables list')
  
  call ESMF_FieldGet(pcpField,localDE=0,farrayPtr=pcp,rc=status)
  call LIS_verify(status)

!- Convert precipitation sum to rate:
   if( LIS_FORC_CRainf%selectOpt .ne. 1 ) then
     do t=1,LIS_rc%ntiles(n)
        index1 = LIS_domain(n)%tile(t)%index
        if (LIS_forc(n,findex)%metdata2(1,index1) .ne. -1.0) then
           pcp(t) = LIS_forc(n,findex)%metdata2(1,index1)
        endif
     enddo

!- Applying a convective rainfall amount to observed precip field: 
   elseif( LIS_FORC_CRainf%selectOpt == 1 ) then

     allocate(ratio(LIS_rc%ntiles(n)))

     call ESMF_StateGet(LIS_FORC_Base_State(n,findex),&
          trim(LIS_FORC_CRainf%varname(1)),cpcpField,&
          rc=status)
     call LIS_verify(status, 'Error: enable CRainf in forcing variables list')
  
     call ESMF_FieldGet(cpcpField,localDE=0,farrayPtr=cpcp,rc=status)
     call LIS_verify(status)
  
  !------------------------------------------------------------------------
  ! Compute ratio between convective model precip and total model precip
  ! so that it can be applied to the observed global precip
  !------------------------------------------------------------------------
     do t = 1,LIS_rc%ntiles(n)
        if( pcp(t) .ne. 0.0 .and.  &
            pcp(t) .ne. LIS_rc%udef .and.  &
           cpcp(t) .ne. LIS_rc%udef) then
          ratio(t) = cpcp(t) / pcp(t)
          if (ratio(t) .gt. 1.0) ratio(t) = 1.0
          if (ratio(t) .lt. 0.0) ratio(t) = 0.0
        else
          ratio(t) = 0.0
        endif
     enddo
  
     do t=1,LIS_rc%ntiles(n)
        index1 = LIS_domain(n)%tile(t)%index
        if (LIS_forc(n,findex)%metdata2(1,index1) .ne. -1.0) then
           pcp(t) = LIS_forc(n,findex)%metdata2(1,index1) 
           cpcp(t) = ratio(t) * pcp(t)
        endif
     enddo
     deallocate(ratio)
   endif

end subroutine timeinterp_RFE2gdas
